table.autosummary.align-default {
    margin-left: 2em;
    margin-right: 0px;
    border: 0px;
    box-shadow: none;
    -webkit-box-shadow: none;
    -moz-box-shadow: none;

    /* turn the table into a flex grid */
    display: grid;
    /* TODO: automate between 1 and 2 columns in the grid */
}

table.autosummary.align-default tbody,tr {
    display: contents;
}

table.autosummary.docutils td {
    border: 0px;
}

table.autosummary.docutils td + td {
    margin-left: 1em;
}
